.select2-container {
    width:100%;
}
.select2-container-multi .select2-choices .select2-search-field input {
    height:auto;
}
.select2-choices {
    border:1px solid #CCCCCC;
    border-radius:3px;
    color:#555555;
    font-size:13px;
    line-height:18px;
    padding:4px;
}
.select2-container-multi .select2-choices li.select2-search-choice {
    margin:4px 0 4px 5px;
}